@@ -4,8 +4,10 @@ from __future__ import absolute_import, division, print_function
 import pytest
 import math
 import os
+import random
 import sys
 from collections import Iterator
+from itertools import repeat
 
 import partd
 from toolz import merge, join, filter, identity, valmap, groupby, pluck
@@ -15,9 +17,8 @@ import dask.bag as db
 from dask.bag.core import (Bag, lazify, lazify_task, map, collect,
                            reduceby, reify, partition, inline_singleton_lists,
                            optimize, from_delayed)
-from dask.async import get_sync
 from dask.compatibility import BZ2File, GzipFile, PY2
-from dask.utils import filetexts, tmpfile, tmpdir, open
+from dask.utils import filetexts, tmpfile, tmpdir
 from dask.utils_test import inc, add
 
 
@@ -47,14 +48,14 @@ def test_keys():
     assert sorted(b._keys()) == sorted(dsk.keys())
 
 
+# Deprecated test for old map behavior
 def test_map():
     c = b.map(inc)
-    expected = merge(dsk, dict(((c.name, i), (reify, (map, inc, (b.name, i))))
-                               for i in range(b.npartitions)))
-    assert c.dask == expected
+    assert c.compute() == list(map(inc, b.compute()))
     assert c.name == b.map(inc).name
 
 
+# Deprecated test for old map behavior
 def test_map_function_with_multiple_arguments():
     b = db.from_sequence([(1, 10), (2, 20), (3, 30)], npartitions=3)
     assert list(b.map(lambda x, y: x + y).compute(get=dask.get)) == [11, 22, 33]
@@ -71,6 +72,7 @@ class B(object):
         pass
 
 
+# Deprecated test for old map behavior
 def test_map_with_constructors():
     assert db.from_sequence([[1, 2, 3]]).map(A).compute()
     assert db.from_sequence([1, 2, 3]).map(B).compute()
@@ -84,6 +86,7 @@ def test_map_with_constructors():
     assert failed
 
 
+# Deprecated test for old map behavior
 def test_map_with_builtins():
     b = db.from_sequence(range(3))
     assert ' '.join(b.map(str)) == '0 1 2'
@@ -98,6 +101,7 @@ def test_map_with_builtins():
         [True, False]
 
 
+# Deprecated test for old map behavior
 def test_map_with_kwargs():
     b = db.from_sequence(range(100), npartitions=10)
     assert b.map(lambda x, factor=0: x * factor,
@@ -109,6 +113,105 @@ def test_map_with_kwargs():
                  factor=2).sum().compute() == 2.0
 
 
+def test_bag_map():
+    b = db.from_sequence(range(100), npartitions=10)
+    b2 = db.from_sequence(range(100, 200), npartitions=10)
+    x = b.compute()
+    x2 = b2.compute()
+
+    def myadd(a=1, b=2, c=3):
+        return a + b + c
+
+    assert db.map(myadd, b).compute() == list(map(myadd, x))
+    assert db.map(myadd, a=b).compute() == list(map(myadd, x))
+    assert db.map(myadd, b, b2).compute() == list(map(myadd, x, x2))
+    assert db.map(myadd, b, 10).compute() == [myadd(i, 10) for i in x]
+    assert db.map(myadd, 10, b=b).compute() == [myadd(10, b=i) for i in x]
+
+    sol = [myadd(i, b=j, c=100) for (i, j) in zip(x, x2)]
+    assert db.map(myadd, b, b=b2, c=100).compute() == sol
+
+    sol = [myadd(i, c=100) for (i, j) in zip(x, x2)]
+    assert db.map(myadd, b, c=100).compute() == sol
+
+    x_sum = sum(x)
+    sol = [myadd(x_sum, b=i, c=100) for i in x2]
+    assert db.map(myadd, b.sum(), b=b2, c=100).compute() == sol
+
+    sol = [myadd(i, b=x_sum, c=100) for i in x2]
+    assert db.map(myadd, b2, b.sum(), c=100).compute() == sol
+
+    sol = [myadd(a=100, b=x_sum, c=i) for i in x2]
+    assert db.map(myadd, a=100, b=b.sum(), c=b2).compute() == sol
+
+    a = dask.delayed(10)
+    assert db.map(myadd, b, a).compute() == [myadd(i, 10) for i in x]
+    assert db.map(myadd, b, b=a).compute() == [myadd(i, b=10) for i in x]
+
+    # Mispatched npartitions
+    fewer_parts = db.from_sequence(range(100), npartitions=5)
+    with pytest.raises(ValueError):
+        db.map(myadd, b, fewer_parts)
+
+    # No bags
+    with pytest.raises(ValueError):
+        db.map(myadd, b.sum(), 1, 2)
+
+    # Unequal partitioning
+    unequal = db.from_sequence(range(110), npartitions=10)
+    with pytest.raises(ValueError):
+        db.map(myadd, b, unequal, c=b2).compute()
+    with pytest.raises(ValueError):
+        db.map(myadd, b, b=unequal, c=b2).compute()
+
+
+def test_map_method():
+    b = db.from_sequence(range(100), npartitions=10)
+    b2 = db.from_sequence(range(100, 200), npartitions=10)
+    x = b.compute()
+    x2 = b2.compute()
+
+    def myadd(a, b=2, c=3):
+        return a + b + c
+
+    assert b.map(myadd).compute() == list(map(myadd, x))
+    assert b.map(myadd, b2).compute() == list(map(myadd, x, x2))
+    assert b.map(myadd, 10).compute() == [myadd(i, 10) for i in x]
+    assert b.map(myadd, b=10).compute() == [myadd(i, b=10) for i in x]
+    assert (b.map(myadd, b2, c=10).compute() ==
+            [myadd(i, j, 10) for (i, j) in zip(x, x2)])
+    x_sum = sum(x)
+    assert (b.map(myadd, b.sum(), c=10).compute() ==
+            [myadd(i, x_sum, 10) for i in x])
+
+    # check that map works with multiarg functions. Can be removed after
+    # deprecated behavior is removed
+    assert b.map(add, b2).compute() == list(map(add, x, x2))
+
+    # check that map works with vararg functions. Can be removed after
+    # deprecated behavior is removed
+    def vararg_inc(*args):
+        return inc(*args)
+
+    assert b.map(vararg_inc).compute(get=dask.get) == list(map(inc, x))
+
+
+def test_starmap():
+    data = [(1, 2), (3, 4), (5, 6), (7, 8), (9, 10)]
+    b = db.from_sequence(data, npartitions=2)
+
+    def myadd(a, b, c=0):
+        return a + b + c
+
+    assert b.starmap(myadd).compute() == [myadd(*a) for a in data]
+    assert b.starmap(myadd, c=10).compute() == [myadd(*a, c=10) for a in data]
+    max_second = b.pluck(1).max()
+    assert (b.starmap(myadd, c=max_second).compute() ==
+            [myadd(*a, c=max_second.compute()) for a in data])
+    c = dask.delayed(10)
+    assert b.starmap(myadd, c=c).compute() == [myadd(*a, c=10) for a in data]
+
+
 def test_filter():
     c = b.filter(iseven)
     expected = merge(dsk, dict(((c.name, i),
@@ -188,7 +291,7 @@ def test_fold():
     assert set(d.fold(lambda a, b: ''.join([a, b]), initial='').compute()) == set('hello')
 
     e = db.from_sequence([[1], [2], [3]], npartitions=2)
-    with dask.set_options(get=get_sync):
+    with dask.set_options(get=dask.get):
         assert set(e.fold(add, initial=[]).compute()) == set([1, 2, 3])
 
 
@@ -229,8 +332,9 @@ def test_topk():
     assert b.topk(4).name == b.topk(4).name
 
 
-def test_topk_with_non_callable_key():
-    b = db.from_sequence([(1, 10), (2, 9), (3, 8)], npartitions=2)
+@pytest.mark.parametrize('npartitions', [1, 2])
+def test_topk_with_non_callable_key(npartitions):
+    b = db.from_sequence([(1, 10), (2, 9), (3, 8)], npartitions=npartitions)
     assert list(b.topk(2, key=1)) == [(1, 10), (2, 9)]
     assert list(b.topk(2, key=0)) == [(3, 8), (2, 9)]
     assert b.topk(2, key=1).name == b.topk(2, key=1).name
@@ -285,15 +389,20 @@ def test_tree_reductions():
     assert c.key != b.sum().key
 
 
-def test_mean():
-    assert b.mean().compute(get=dask.get) == 2.0
-    assert float(b.mean()) == 2.0
+@pytest.mark.parametrize('npartitions', [1, 3, 4])
+def test_aggregation(npartitions):
+    L = list(range(15))
+    b = db.range(15, npartitions=npartitions)
+    assert b.mean().compute(get=dask.get) == sum(L) / len(L)
+    assert b.sum().compute(get=dask.get) == sum(L)
+    assert b.count().compute(get=dask.get) == len(L)
 
 
-def test_non_splittable_reductions():
+@pytest.mark.parametrize('npartitions', [1, 10])
+def test_non_splittable_reductions(npartitions):
     np = pytest.importorskip('numpy')
     data = list(range(100))
-    c = db.from_sequence(data, npartitions=10)
+    c = db.from_sequence(data, npartitions=npartitions)
     assert c.mean().compute() == np.mean(data)
     assert c.std().compute(get=dask.get) == np.std(data)
 
@@ -332,18 +441,35 @@ def test_map_partitions():
     assert b.map_partitions(lambda a: len(a) + 1).name != b.map_partitions(len).name
 
 
-def test_map_partitions_with_kwargs():
-    b = db.from_sequence(range(100), npartitions=10)
-    assert b.map_partitions(
-        lambda X, factor=0: [x * factor for x in X],
-        factor=2).sum().compute() == 9900.0
-    assert b.map_partitions(
-        lambda X, total=0: [x / total for x in X],
-        total=b.sum()).sum().compute() == 1.0
-    assert b.map_partitions(
-        lambda X, factor=0, total=0: [x * factor / total for x in X],
-        total=b.sum(),
-        factor=2).sum().compute() == 2.0
+def test_map_partitions_args_kwargs():
+    x = [random.randint(-100, 100) for i in range(100)]
+    y = [random.randint(-100, 100) for i in range(100)]
+
+    dx = db.from_sequence(x, npartitions=10)
+    dy = db.from_sequence(y, npartitions=10)
+
+    def maximum(x, y=0):
+        y = repeat(y) if isinstance(y, int) else y
+        return [max(a, b) for (a, b) in zip(x, y)]
+
+    sol = maximum(x, y=10)
+    assert db.map_partitions(maximum, dx, y=10).compute() == sol
+    assert dx.map_partitions(maximum, y=10).compute() == sol
+    assert dx.map_partitions(maximum, 10).compute() == sol
+
+    sol = maximum(x, y)
+    assert db.map_partitions(maximum, dx, dy).compute() == sol
+    assert dx.map_partitions(maximum, y=dy).compute() == sol
+    assert dx.map_partitions(maximum, dy).compute() == sol
+
+    dy_mean = dy.mean().apply(int)
+    sol = maximum(x, int(sum(y) / len(y)))
+    assert dx.map_partitions(maximum, y=dy_mean).compute() == sol
+    assert dx.map_partitions(maximum, dy_mean).compute() == sol
+
+    dy_mean = dask.delayed(dy_mean)
+    assert dx.map_partitions(maximum, y=dy_mean).compute() == sol
+    assert dx.map_partitions(maximum, dy_mean).compute() == sol
 
 
 def test_random_sample_size():
@@ -630,13 +756,14 @@ def test_concat():
     b = db.from_sequence([4, 5, 6])
     c = db.concat([a, b])
     assert list(c) == [1, 2, 3, 4, 5, 6]
-
     assert c.name == db.concat([a, b]).name
-    assert b.concat().name != a.concat().name
-    assert b.concat().name == b.concat().name
 
-    b = db.from_sequence([1, 2, 3]).map(lambda x: x * [1, 2, 3])
-    assert list(b.concat()) == [1, 2, 3] * sum([1, 2, 3])
+
+def test_flatten():
+    b = db.from_sequence([[1], [2, 3]])
+    assert list(b.flatten()) == [1, 2, 3]
+    assert list(b.flatten()) == list(b.concat())
+    assert b.flatten().name == b.flatten().name
 
 
 def test_concat_after_map():
@@ -875,6 +1002,23 @@ def test_to_delayed():
     assert t.compute() == 21
 
 
+def test_to_delayed_optimizes():
+    b = db.from_sequence([1, 2, 3, 4, 5, 6], npartitions=1)
+    b2 = b.map(inc).map(inc).map(inc)
+
+    [d] = b2.to_delayed()
+    text = str(dict(d.dask))
+    assert text.count('reify') == 1
+
+    d = b2.sum().to_delayed()
+    text = str(dict(d.dask))
+    assert text.count('reify') == 0
+
+    [d] = b2.to_textfiles('foo.txt', compute=False)
+    text = str(dict(d.dask))
+    assert text.count('reify') <= 0
+
+
 def test_from_delayed():
     from dask.delayed import delayed
     a, b, c = delayed([1, 2, 3]), delayed([4, 5, 6]), delayed([7, 8, 9])
@@ -900,7 +1044,7 @@ def test_from_delayed_iterator():
     assert db.compute(
         bag.count(),
         bag.pluck('operations').count(),
-        bag.pluck('operations').concat().count(),
+        bag.pluck('operations').flatten().count(),
         get=dask.get,
     ) == (25, 25, 50)
 
@@ -1035,13 +1179,14 @@ def test_reduction_empty():
     assert b.filter(lambda x: x % 2 == 0).min().compute(get=dask.get) == 0
 
 
-def test_reduction_empty_aggregate():
-    b = db.from_sequence([0, 0, 0, 1], npartitions=4).filter(None)
+@pytest.mark.parametrize('npartitions', [1, 2, 4])
+def test_reduction_empty_aggregate(npartitions):
+    b = db.from_sequence([0, 0, 0, 1], npartitions=npartitions).filter(None)
     assert b.min(split_every=2).compute(get=dask.get) == 1
     vals = db.compute(b.min(split_every=2), b.max(split_every=2), get=dask.get)
     assert vals == (1, 1)
     with pytest.raises(ValueError):
-        b = db.from_sequence([0, 0, 0, 0], npartitions=4)
+        b = db.from_sequence([0, 0, 0, 0], npartitions=npartitions)
         b.filter(None).min(split_every=2).compute(get=dask.get)
 
 
@@ -1088,7 +1233,7 @@ def test_bag_picklable():
 
 def test_msgpack_unicode():
     b = db.from_sequence([{"a": 1}]).groupby("a")
-    result = b.compute(get=dask.async.get_sync)
+    result = b.compute(get=dask.get)
     assert dict(result) == {1: [{'a': 1}]}
 
 
@@ -1142,3 +1287,11 @@ def test_temporary_directory():
         b2 = b.groupby(lambda x: x % 2)
         b2.compute()
         assert any(fn.endswith('.partd') for fn in os.listdir(os.getcwd()))
+
+
+def test_empty_bag():
+    b = db.from_sequence([])
+    assert b.map(inc).all().compute(get=dask.get)
+    assert not b.map(inc).any().compute(get=dask.get)
+    assert not b.map(inc).sum().compute(get=dask.get)
+    assert not b.map(inc).count().compute(get=dask.get)
