@@ -4,7 +4,6 @@ from math import ceil
 from operator import getitem
 import os
 from threading import Lock
-from warnings import warn
 
 import pandas as pd
 import numpy as np
@@ -123,7 +122,7 @@ def from_pandas(data, npartitions=None, chunksize=None, sort=True, name=None):
         the size and index of the dataframe, the output may have fewer
         partitions than requested.
     chunksize : int, optional
-        The size of the partitions of the index.
+        The number of rows per index partition to use.
     sort: bool
         Sort input first to obtain cleanly divided partitions or don't sort and
         don't get cleanly divided partitions
@@ -210,7 +209,7 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, lock=lock,
     ----------
     x : bcolz.ctable
     chunksize : int, optional
-        The size of blocks to pull out from ctable.
+        The size(rows) of blocks to pull out from ctable.
     categorize : bool, defaults to True
         Automatically categorize all string dtypes
     index : string, optional
@@ -470,8 +469,7 @@ def to_records(df):
 
 
 @insert_meta_param_description
-def from_delayed(dfs, meta=None, divisions=None, prefix='from-delayed',
-                 metadata=None):
+def from_delayed(dfs, meta=None, divisions=None, prefix='from-delayed'):
     """ Create Dask DataFrame from many Dask Delayed objects
 
     Parameters
@@ -490,9 +488,6 @@ def from_delayed(dfs, meta=None, divisions=None, prefix='from-delayed',
     prefix : str, optional
         Prefix to prepend to the keys.
     """
-    if metadata is not None and meta is None:
-        warn("Deprecation warning: Use meta keyword, not metadata")
-        meta = metadata
     from dask.delayed import Delayed
     if isinstance(dfs, Delayed):
         dfs = [dfs]
