@@ -6,9 +6,9 @@ from random import random
 from toolz import identity, partial, merge
 import pytest
 
-from dask import set_options
+from dask import set_options, compute
 from dask.compatibility import PY2, PY3
-from dask.delayed import delayed, to_task_dask, compute, Delayed
+from dask.delayed import delayed, to_task_dask, Delayed
 
 
 def test_to_task_dask():
@@ -403,7 +403,7 @@ def test_callable_obj():
     assert f().compute() == 2
 
 
-def test_name_consitent_across_instances():
+def test_name_consistent_across_instances():
     func = delayed(identity, pure=True)
 
     data = {'x': 1, 'y': 25, 'z': [1, 2, 3]}
